
//package LineLeader;

import lejos.nxt.*;
import lejos.nxt.comm.*;
import java.io.*;
//import lejos.robotics.*;
//import lejos.nxt.addon.*;
import lejos.robotics.navigation.*;
import javax.microedition.lcdui.Graphics;
import lejos.robotics.proposal.DifferentialPilot;
import lejos.nxt.LCD;
import lejos.nxt.addon.NXTLineLeader;
import lejos.robotics.navigation.TachoPilot;
import lejos.nxt.addon.CompassSensor;
import lejos.robotics.navigation.CompassPilot;


public class ArtBot {
 	
 	static Pilot pilot = new TachoPilot(8.16f * 1.012f, 19.75f * 1.041f, Motor.B, Motor.C, true);
 	
 	static Motor Pen = Motor.A;
 	static boolean PenIsDown = true;	// pen always starts in the down position
 
	public static void main(String[] args) throws Exception
	{
		init();
		followInstructions();
	}
	
	private static void init()
	{
		//pilot.setSpeed(50);	// cm/s	// deprecated
		//pilot.regulateSpeed(false);	// it is useful to turn this off for line followers // doesn't work
		pilot.setMoveSpeed(550);		// cm/s
		pilot.setTurnSpeed(350);		// cm/s ?
		Pen.setSpeed(300);
	}
	
	private static void penDown() { penDown(true); }
	private static void penUp()   { penDown(false); }
	
	private static void penDown(boolean put_it_down)
	{
		if (PenIsDown != put_it_down)
		{
			Pen.rotate(180);
			PenIsDown = put_it_down;
		}
	}

	public static void followInstructions()
	{
		// the robot receives instructions over Bluetooth and acts upon them
		
		LCD.clear();
		System.out.println("Awaiting connection...");
		
		BTConnection btc = Bluetooth.waitForConnection();
		System.out.println("Connected.  Awaiting instructions.");

		DataInputStream dis = btc.openDataInputStream();
		DataOutputStream dos = btc.openDataOutputStream();

		boolean time_to_quit = false;
		
		while (!time_to_quit)
		{
			int command_code = -1;
			float f1 = 0, f2 = 0;
			
			// receive a message -- it should be a command string
			// and two floating-point parameters
			try
			{
				//command = dis.readUTF(); // unimplemented on NXT
				command_code = dis.readInt();
				f1 = dis.readFloat();
				f2 = dis.readFloat();
			}
			catch (IOException ioe)
			{
				System.out.println("Failure to receive command.");
			}
			
			if (command_code != -1)
			{
				time_to_quit = actOnCommand(command_code, f1, f2);
			}
			
			// send a signal to say we are ready for a new command
			//dos.writeInt(1);
		}
	}
	
	public static boolean actOnCommand(int command_code, float f1, float f2)
	{
		// this function acts on a command
		// it returns true if the command indicates
		// that we should quit
		
		System.out.println(command_code + "(" + f1 + ", " + f2 + ")");
		
		switch(command_code)
		{
			case 0:		// heartbeat
				return false;
			
			case 1:		// quit
				return true;
			
			case 2:		// travel (f1 = distance)
				pilot.travel(f1);
				return false;
			
			case 3:		// rotate (f1 = degrees)
				pilot.rotate(f1);
				return false;
			
			case 4:		// play note (f1 = freq, f2 = duration (in ms))
				Sound.playNote(Sound.XYLOPHONE, (int)f1, (int)f2);
				return false;
			
			case 5:		// set pen status (f1 = 1 for down, otherwise, up)
				penDown(f1 == 1.0f);
				return false;
		}
		
		return false;
	}

	
	public static void delay_ms(int milliseconds)
	{
		try { Thread.sleep(milliseconds); } catch (Exception e) {}
	}

}
