import lejos.pc.comm.*;
import java.io.*;
import java.util.HashMap;
import java.util.Map;

// This code started with the BTSend.java sample project,
// created by Lawrie Griffiths, included with LeJOS NXJ 0.85, 
//
// It also makes use of the ScratchIO java libraries by
// Andrew Davison, March 2009, ad@fivedots.coe.psu.ac.th.
// See http://fivedots.coe.psu.ac.th/~ad/jg/ch137/index.html.
//
// Additions were made by Clinton Blackmore.
//
// Compile by running 'nxjpcc *.java' and run by using 'nxjpc TalkToBot'

public class TalkToBot
{
    public static void main(String[] args)
    {
        // Create the map of commands to command codes
        Map<String, Integer> command_map = new HashMap<String, Integer>();
        
        command_map.put("heartbeat", 0);
        command_map.put("quit",      1);
        command_map.put("travel",    2);
        command_map.put("rotate",    3);
        command_map.put("playnote",  4);
        command_map.put("pendown",   5);
        
        
        ScratchIO sio = new ScratchIO();
        System.out.println("Connected to Scratch.");
           
        NXTConnector conn = new NXTConnector();

        // Connect to any NXT over Bluetooth
        System.out.print("Establishing connection to NXT. ");
        
        
        boolean connected = conn.connectTo("btspp://");

        if (!connected) {
            System.err.println("Failed to connect to any NXT");
            System.exit(1);
        }
        else
        {
            System.out.println("Success.");
        }

        DataOutputStream dos = conn.getDataOut();
        DataInputStream dis = conn.getDataIn();
        

        boolean time_to_quit = false;

        while (!time_to_quit)
        {    
            ScratchMessage scratchMsg = sio.readMsg();
            
            // We are interested in messages in this format:
            // ">command float1 float2"
            // such as ">goto 10 12.4"
            
            // In this code, I only care about broadcast messages that start with '>'
            if (scratchMsg != null &&
                scratchMsg.getMessageType() == ScratchMessage.BROADCAST_MSG)
            {
                String broadcast = scratchMsg.getName();
                if (broadcast.charAt(0) == '>')
                {
                    String[] params = StringToks.parseTokens(broadcast.substring(1));
                    
                    try
                    {
                        String command = params[0];
                        if (command_map.containsKey(command))
                        {
                            Integer command_code = command_map.get(command);
                            float f1 = Float.valueOf(params[1]).floatValue();
                            float f2 = Float.valueOf(params[2]).floatValue();
                            
                            // Now, send a message to the NXT
        
                            System.out.println(command + "(" + f1 + ", " + f2 + ")");
                            
                            
                            try
                            {
                                //dos.writeUTF(command);    // not implements on NXT
                                dos.writeInt(command_code);
                                dos.writeFloat(f1);
                                dos.writeFloat(f1);
                                dos.flush();                            
                            }
                            catch (IOException ioe)
                            {
                                System.out.println("-- Failed to send to NXT.");
                            }
                            
                            
                            // Now, wait for the NXT to be ready
                            
                            
                            // Stop execution if this was the quit message
                            if (command == "quit")
                            {
                                time_to_quit = true;
                            }
                        }
                        else
                        {
                            System.out.println("Unknown command issued: " + command);
                        }
                    }
                    catch (Exception e)
                    {
                        System.out.println("Received unrecognized command: '" + broadcast + "'");
                    }
                }
                else
                {
                    System.out.println(broadcast);
                }
            }            
        }

        try
        {
            dis.close();
            dos.close();
            conn.close();
            System.out.println("Cleanly terminated connection with NXT");
        }
        catch (IOException ioe)
        {
            System.out.println("IOException closing connection:");
            System.out.println(ioe.getMessage());
        }
    }
}
